@@ -1,24 +1,30 @@
 /**
- * @package		HUBzero
- * @author		Rick Kennell <kennell@purdue.edu>
- * @copyright	Copyright 2005-2009 by Purdue Research Foundation, West Lafayette, IN 47906
- * @license		http://www.gnu.org/licenses/gpl-2.0.html GPLv2
+ * @package      hubzero-usermap
+ * @file         usermap.c
+ * @author       Rick Kennell <kennell@purdue.edu>
+ * @copyright    Copyright (c) 2005-2010 Purdue University. All rights reserved.
+ * @license      http://www.gnu.org/licenses/lgpl-3.0.html LGPLv3
  *
- * Copyright 2005-2009 by Purdue Research Foundation, West Lafayette, IN 47906.
+ * Copyright (c) 2005-2010 Purdue University
  * All rights reserved.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License,
- * version 2 as published by the Free Software Foundation.
+ * This file is part of: The HUBzero(R) Platform for Scientific Collaboration
  *
- * This program is distributed in the hope that it will be useful,
+ * The HUBzero(R) Platform for Scientific Collaboration (HUBzero) is free
+ * software: you can redistribute it and/or modify it under the terms of
+ * the GNU Lesser General Public License as published by the Free Software
+ * Foundation, either version 3 of the License, or (at your option) any
+ * later version.
+ *
+ * HUBzero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * HUBzero is a registered trademark of Purdue University.
  */
 
 #ifdef linux
@@ -478,13 +484,6 @@ int main(int argc, char *argv[])
     char * source_dir = strdup(argv[1]);
     const char * target_dir = argv[2];
 
-    // Treat the source_dir as a username.  If it's a path, strip all but
-    // the last element (assumedly the username).
-    char *p = strrchr(source_dir,'/');
-    if (p != NULL) {
-      memmove(source_dir, &p[1], strlen(&p[1])+1);
-    }
-
     openlog("usermap", 0, LOG_DAEMON);
 
     if ((argc < 3) || (argv[1][0] == '-') || (argv[2][0] == '1')) {
@@ -606,13 +605,16 @@ int main(int argc, char *argv[])
     return 0;
 #endif
 
-    if (source_user[0] != '\0') {
-        if (strcmp(source_user,source_dir) != 0) {
-            error("source_user does not match source_dir.");
-            return 1;
+    if (source_dir[0] != '/') {
+        // Treat the source_dir as a username.
+        if (source_user[0] != '\0') {
+            if (strcmp(source_user,source_dir) != 0) {
+                error("source_user does not match source_dir.");
+                return 1;
+            }
+        } else {
+            strcpy(source_user, source_dir);
         }
-    } else {
-        strcpy(source_user, source_dir);
     }
 
     struct passwd *passwd = getpwnam(source_user);
@@ -622,8 +624,10 @@ int main(int argc, char *argv[])
     }
     source_uid = passwd->pw_uid;
     source_gid = passwd->pw_gid;
-    free(source_dir);
-    source_dir = strdup(passwd->pw_dir);
+    if (source_dir[0] != '/') {
+        free(source_dir);
+        source_dir = strdup(passwd->pw_dir);
+    }
 
     // Make sure that source_dir exists...
     struct stat stat_buf;
@@ -665,6 +669,14 @@ int main(int argc, char *argv[])
         return 1;
     }
 
+    // We must initialize the group list BEFORE chroot.
+    // Otherwise we won't be able to see the authentication entities.
+    res = initgroups(source_user, source_gid);
+    if (res < 0) {
+        perror("initgroups");
+        return 1;
+    }
+
     res = chroot(source_dir);
     if (res < 0) {
         perror("chroot");
